<head><link href="/css/app.ada508f9934857d427a3.css" rel="stylesheet"></head><!doctype html>
<title>Spintop-OpenHTF</title>

<!-- TODO(Kenadia): sockjs-client should be included via node. -->
<link rel="icon" type="image/x-icon" href="img/spintop-icon.png">
<script src="https://cdnjs.cloudflare.com/ajax/libs/sockjs-client/1.1.4/sockjs.min.js"></script>

<base href="/">
<htf-app config="{{ json_encode(config) }}">Loading...</htf-app>
<script type="text/javascript" src="/js/polyfills.ada508f9934857d427a3.js"></script><script type="text/javascript" src="/js/vendor.ada508f9934857d427a3.js"></script><script type="text/javascript" src="/js/app.ada508f9934857d427a3.js"></script>