* {
  box-sizing: border-box; }

body {
  background: #f6f6f6;
  color: #323232;
  font-family: Helvetica, Arial, sans-serif;
  font-size: 14px; }

h1,
h2,
h3,
h4,
h5,
h6 {
  font-weight: initial;
  margin: initial; }

a:link,
a:visited {
  color: #07f;
  text-decoration: underline;
  transition: 300ms ease color; }
  a:link:hover, a:link:focus, a:link:active,
  a:visited:hover,
  a:visited:focus,
  a:visited:active {
    color: #0053b3; }
  a:link:disabled,
  a:visited:disabled {
    color: #949a9f;
    cursor: initial; }

input[type='text'] {
  border-radius: 3px;
  border: 1px solid #cccccc;
  box-shadow: 0 0 10px rgba(0, 0, 0, 0.05) inset;
  font-size: 16px;
  height: 40px;
  padding: 0 12px;
  transition: 300ms ease border-color;
  width: 100%; }
  input[type='text']:focus {
    border-color: rgba(0, 119, 255, 0.5);
    outline: none;
    animation: htf-base-input-pulse 2s infinite; }

@keyframes htf-base-input-pulse {
  0% {
    box-shadow: 0 0 0 0 rgba(0, 119, 255, 0.25), 0 0 10px rgba(0, 0, 0, 0.1) inset; }
  70% {
    box-shadow: 0 0 0 8px rgba(0, 119, 255, 0), 0 0 10px rgba(0, 0, 0, 0.1) inset; }
  100% {
    box-shadow: 0 0 0 0 rgba(0, 119, 255, 0), 0 0 10px rgba(0, 0, 0, 0.1) inset; } }

.htf-rounded-button-blue {
  -webkit-appearance: initial;
  background: transparent;
  border: 0;
  padding: 0;
  background: #07f;
  border-radius: 16px;
  border: 1px solid #07f;
  color: #fff;
  cursor: pointer;
  display: inline-block;
  font-size: 12px;
  height: 32px;
  padding: 0 20px;
  text-transform: uppercase;
  transition: 300ms ease border-color, 300ms ease background; }
  .htf-rounded-button-blue:focus, .htf-rounded-button-blue:active {
    outline: none; }
  .htf-rounded-button-blue:focus {
    border-color: #005fcc; }
  .htf-rounded-button-blue:active, .htf-rounded-button-blue:hover {
    background: #4da0ff;
    border-color: #4da0ff; }
  .htf-rounded-button-blue:disabled {
    background: #80bbff;
    border-color: #80bbff; }

.htf-rounded-button-red {
  -webkit-appearance: initial;
  background: transparent;
  border: 0;
  padding: 0;
  background: #ff5d4e;
  border-radius: 16px;
  border: 1px solid #ff5d4e;
  color: #fff;
  cursor: pointer;
  display: inline-block;
  font-size: 12px;
  height: 32px;
  padding: 0 20px;
  text-transform: uppercase;
  transition: 300ms ease border-color, 300ms ease background; }
  .htf-rounded-button-red:focus, .htf-rounded-button-red:active {
    outline: none; }
  .htf-rounded-button-red:focus {
    border-color: #ff2e1b; }
  .htf-rounded-button-red:active, .htf-rounded-button-red:hover {
    background: #ffa39b;
    border-color: #ffa39b; }
  .htf-rounded-button-red:disabled {
    background: #ffd2ce;
    border-color: #ffd2ce; }

.htf-rounded-button-grey {
  -webkit-appearance: initial;
  background: transparent;
  border: 0;
  padding: 0;
  background: #e5e5e5;
  border-radius: 16px;
  border: 1px solid #e5e5e5;
  color: #323232;
  cursor: pointer;
  display: inline-block;
  font-size: 12px;
  height: 32px;
  padding: 0 20px;
  text-transform: uppercase;
  transition: 300ms ease border-color, 300ms ease background;
  font-weight: bold; }
  .htf-rounded-button-grey:focus, .htf-rounded-button-grey:active {
    outline: none; }
  .htf-rounded-button-grey:focus {
    border-color: #cccccc; }
  .htf-rounded-button-grey:active, .htf-rounded-button-grey:hover {
    background: white;
    border-color: white; }
  .htf-rounded-button-grey:disabled {
    background: white;
    border-color: white; }
  .htf-rounded-button-grey:active, .htf-rounded-button-grey:hover {
    background: #cccccc; }

.htf-link-button {
  -webkit-appearance: initial;
  background: transparent;
  border: 0;
  padding: 0;
  color: #07f;
  text-decoration: underline;
  transition: 300ms ease color;
  font-size: 12px;
  cursor: pointer; }
  .htf-link-button:focus, .htf-link-button:active {
    outline: none; }
  .htf-link-button:hover, .htf-link-button:focus, .htf-link-button:active {
    color: #0053b3; }
  .htf-link-button:disabled {
    color: #949a9f;
    cursor: initial; }

.htf-support-text {
  color: #949a9f;
  font-size: 12px; }

.htf-status-indicator {
  background: rgba(50, 50, 50, 0.1);
  font-size: 12px;
  font-weight: bold;
  height: 100%;
  margin-left: 15px;
  margin-right: -15px;
  min-width: 100px;
  text-align: center;
  width: 100px; }
  .htf-status-indicator.ng-status-fail {
    background: rgba(255, 93, 78, 0.1);
    color: #ff2e1b; }
  .htf-status-indicator.ng-status-pass {
    background: rgba(0, 232, 157, 0.1);
    color: #00b57a; }
  .htf-status-indicator.ng-status-running {
    background: rgba(0, 119, 255, 0.1);
    color: #005fcc; }
  .htf-status-indicator.ng-status-warning {
    background: rgba(255, 229, 77, 0.18);
    color: #e6c400; }

.ng-tooltip-host {
  position: relative;
  cursor: pointer; }

.ng-tooltip {
  background: rgba(0, 0, 0, 0.7);
  border-radius: 3px;
  color: #fff;
  font-size: 12px;
  line-height: initial;
  max-width: 500px;
  opacity: 0;
  padding: 8px;
  pointer-events: none;
  position: absolute;
  right: -15px;
  top: 50%;
  transform: translateX(100%) translateY(-50%);
  transition: 200ms ease opacity;
  word-wrap: break-word; }
  .ng-tooltip.ng-tooltip--is-visible {
    opacity: 1; }
  .ng-tooltip::before {
    border-bottom: 6px solid transparent;
    border-right: 6px solid rgba(0, 0, 0, 0.7);
    border-top: 6px solid transparent;
    content: '';
    height: 0;
    left: 0;
    position: absolute;
    top: 50%;
    transform: translateX(-100%) translateY(-50%);
    width: 0; }

.form-group {
  margin: 10px; }

.control-label {
  font-style: italic; }

.text-danger {
  color: #ff5d4e; }

.htf-layout-nav-container {
  background: #fff;
  border-bottom: 1px solid #ccc;
  height: 52px;
  line-height: 53px;
  margin-bottom: 15px;
  width: 100%; }

.htf-layout-nav {
  margin-left: auto;
  margin-right: auto;
  max-width: 1000px;
  width: 95%;
  -ms-flex-align: baseline;
      align-items: baseline;
  display: -ms-flexbox;
  display: flex; }

.htf-layout-nav-brand {
  height: 52px; }

.htf-layout-container {
  margin-left: auto;
  margin-right: auto;
  max-width: 1000px;
  width: 95%; }

.htf-layout-widgets-container {
  display: -ms-flexbox;
  display: flex; }

.htf-layout-widgets-row {
  margin-right: -25px; }
  .htf-layout-widgets-row > * {
    -ms-flex: 1;
        flex: 1; }
  .htf-layout-widgets-row .htf-layout-widget {
    margin-right: 25px; }

.htf-layout-widgets-left {
  word-break: break-all;
  -ms-flex-positive: 1;
      flex-grow: 1;
  min-width: 0; }

.htf-layout-widgets-right {
  margin-left: 25px;
  vertical-align: top;
  min-width: 300px;
  max-width: 300px; }

.htf-layout-widget {
  background: #fff;
  border-radius: 3px;
  border: 1px solid #ccc;
  margin-bottom: 25px; }

.htf-layout-header {
  -ms-flex-align: center;
      align-items: center;
  display: -ms-flexbox;
  display: flex;
  height: 43px;
  line-height: 42px;
  padding: 0 15px; }

.htf-layout-widget-header {
  -ms-flex-align: center;
      align-items: center;
  border-bottom: 1px solid #ccc;
  display: -ms-flexbox;
  display: flex;
  height: 54px;
  padding: 0 15px; }
  .htf-layout-widget-header > :first-child {
    font-weight: bold; }

.htf-layout-widget-body {
  padding: 15px; }
  .htf-layout-widget-body + .htf-layout-widget-body {
    border-top: 1px solid #ccc; }

.htf-layout-widget-list {
  list-style: none;
  margin: 0;
  padding: 0; }
  .htf-layout-widget-list > li {
    border-bottom: 1px solid #ccc; }
    .htf-layout-widget-list > li:last-of-type {
      border-bottom: 0; }

.htf-layout-widget-button {
  border-radius: 0;
  border: 0;
  color: #ff5d4e;
  width: 100%; }

/*! normalize.css v7.0.0 | MIT License | github.com/necolas/normalize.css */
/* Document
   ========================================================================== */
/**
 * 1. Correct the line height in all browsers.
 * 2. Prevent adjustments of font size after orientation changes in
 *    IE on Windows Phone and in iOS.
 */
html {
  line-height: 1.15;
  /* 1 */
  -ms-text-size-adjust: 100%;
  /* 2 */
  -webkit-text-size-adjust: 100%;
  /* 2 */ }

/* Sections
   ========================================================================== */
/**
 * Remove the margin in all browsers (opinionated).
 */
body {
  margin: 0; }

/**
 * Add the correct display in IE 9-.
 */
article,
aside,
footer,
header,
nav,
section {
  display: block; }

/**
 * Correct the font size and margin on `h1` elements within `section` and
 * `article` contexts in Chrome, Firefox, and Safari.
 */
h1 {
  font-size: 2em;
  margin: 0.67em 0; }

/* Grouping content
   ========================================================================== */
/**
 * Add the correct display in IE 9-.
 * 1. Add the correct display in IE.
 */
figcaption,
figure,
main {
  /* 1 */
  display: block; }

/**
 * Add the correct margin in IE 8.
 */
figure {
  margin: 1em 40px; }

/**
 * 1. Add the correct box sizing in Firefox.
 * 2. Show the overflow in Edge and IE.
 */
hr {
  box-sizing: content-box;
  /* 1 */
  height: 0;
  /* 1 */
  overflow: visible;
  /* 2 */ }

/**
 * 1. Correct the inheritance and scaling of font size in all browsers.
 * 2. Correct the odd `em` font sizing in all browsers.
 */
pre {
  font-family: monospace, monospace;
  /* 1 */
  font-size: 1em;
  /* 2 */ }

/* Text-level semantics
   ========================================================================== */
/**
 * 1. Remove the gray background on active links in IE 10.
 * 2. Remove gaps in links underline in iOS 8+ and Safari 8+.
 */
a {
  background-color: transparent;
  /* 1 */
  -webkit-text-decoration-skip: objects;
  /* 2 */ }

/**
 * 1. Remove the bottom border in Chrome 57- and Firefox 39-.
 * 2. Add the correct text decoration in Chrome, Edge, IE, Opera, and Safari.
 */
abbr[title] {
  border-bottom: none;
  /* 1 */
  text-decoration: underline;
  /* 2 */
  text-decoration: underline dotted;
  /* 2 */ }

/**
 * Prevent the duplicate application of `bolder` by the next rule in Safari 6.
 */
b,
strong {
  font-weight: inherit; }

/**
 * Add the correct font weight in Chrome, Edge, and Safari.
 */
b,
strong {
  font-weight: bolder; }

/**
 * 1. Correct the inheritance and scaling of font size in all browsers.
 * 2. Correct the odd `em` font sizing in all browsers.
 */
code,
kbd,
samp {
  font-family: monospace, monospace;
  /* 1 */
  font-size: 1em;
  /* 2 */ }

/**
 * Add the correct font style in Android 4.3-.
 */
dfn {
  font-style: italic; }

/**
 * Add the correct background and color in IE 9-.
 */
mark {
  background-color: #ff0;
  color: #000; }

/**
 * Add the correct font size in all browsers.
 */
small {
  font-size: 80%; }

/**
 * Prevent `sub` and `sup` elements from affecting the line height in
 * all browsers.
 */
sub,
sup {
  font-size: 75%;
  line-height: 0;
  position: relative;
  vertical-align: baseline; }

sub {
  bottom: -0.25em; }

sup {
  top: -0.5em; }

/* Embedded content
   ========================================================================== */
/**
 * Add the correct display in IE 9-.
 */
audio,
video {
  display: inline-block; }

/**
 * Add the correct display in iOS 4-7.
 */
audio:not([controls]) {
  display: none;
  height: 0; }

/**
 * Remove the border on images inside links in IE 10-.
 */
img {
  border-style: none; }

/**
 * Hide the overflow in IE.
 */
svg:not(:root) {
  overflow: hidden; }

/* Forms
   ========================================================================== */
/**
 * 1. Change the font styles in all browsers (opinionated).
 * 2. Remove the margin in Firefox and Safari.
 */
button,
input,
optgroup,
select,
textarea {
  font-family: sans-serif;
  /* 1 */
  font-size: 100%;
  /* 1 */
  line-height: 1.15;
  /* 1 */
  margin: 0;
  /* 2 */ }

/**
 * Show the overflow in IE.
 * 1. Show the overflow in Edge.
 */
button,
input {
  /* 1 */
  overflow: visible; }

/**
 * Remove the inheritance of text transform in Edge, Firefox, and IE.
 * 1. Remove the inheritance of text transform in Firefox.
 */
button,
select {
  /* 1 */
  text-transform: none; }

/**
 * 1. Prevent a WebKit bug where (2) destroys native `audio` and `video`
 *    controls in Android 4.
 * 2. Correct the inability to style clickable types in iOS and Safari.
 */
button,
html [type="button"],
[type="reset"],
[type="submit"] {
  -webkit-appearance: button;
  /* 2 */ }

/**
 * Remove the inner border and padding in Firefox.
 */
button::-moz-focus-inner,
[type="button"]::-moz-focus-inner,
[type="reset"]::-moz-focus-inner,
[type="submit"]::-moz-focus-inner {
  border-style: none;
  padding: 0; }

/**
 * Restore the focus styles unset by the previous rule.
 */
button:-moz-focusring,
[type="button"]:-moz-focusring,
[type="reset"]:-moz-focusring,
[type="submit"]:-moz-focusring {
  outline: 1px dotted ButtonText; }

/**
 * Correct the padding in Firefox.
 */
fieldset {
  padding: 0.35em 0.75em 0.625em; }

/**
 * 1. Correct the text wrapping in Edge and IE.
 * 2. Correct the color inheritance from `fieldset` elements in IE.
 * 3. Remove the padding so developers are not caught out when they zero out
 *    `fieldset` elements in all browsers.
 */
legend {
  box-sizing: border-box;
  /* 1 */
  color: inherit;
  /* 2 */
  display: table;
  /* 1 */
  max-width: 100%;
  /* 1 */
  padding: 0;
  /* 3 */
  white-space: normal;
  /* 1 */ }

/**
 * 1. Add the correct display in IE 9-.
 * 2. Add the correct vertical alignment in Chrome, Firefox, and Opera.
 */
progress {
  display: inline-block;
  /* 1 */
  vertical-align: baseline;
  /* 2 */ }

/**
 * Remove the default vertical scrollbar in IE.
 */
textarea {
  overflow: auto; }

/**
 * 1. Add the correct box sizing in IE 10-.
 * 2. Remove the padding in IE 10-.
 */
[type="checkbox"],
[type="radio"] {
  box-sizing: border-box;
  /* 1 */
  padding: 0;
  /* 2 */ }

/**
 * Correct the cursor style of increment and decrement buttons in Chrome.
 */
[type="number"]::-webkit-inner-spin-button,
[type="number"]::-webkit-outer-spin-button {
  height: auto; }

/**
 * 1. Correct the odd appearance in Chrome and Safari.
 * 2. Correct the outline style in Safari.
 */
[type="search"] {
  -webkit-appearance: textfield;
  /* 1 */
  outline-offset: -2px;
  /* 2 */ }

/**
 * Remove the inner padding and cancel buttons in Chrome and Safari on macOS.
 */
[type="search"]::-webkit-search-cancel-button,
[type="search"]::-webkit-search-decoration {
  -webkit-appearance: none; }

/**
 * 1. Correct the inability to style clickable types in iOS and Safari.
 * 2. Change font properties to `inherit` in Safari.
 */
::-webkit-file-upload-button {
  -webkit-appearance: button;
  /* 1 */
  font: inherit;
  /* 2 */ }

/* Interactive
   ========================================================================== */
/*
 * Add the correct display in IE 9-.
 * 1. Add the correct display in Edge, IE, and Firefox.
 */
details,
menu {
  display: block; }

/*
 * Add the correct display in all browsers.
 */
summary {
  display: list-item; }

/* Scripting
   ========================================================================== */
/**
 * Add the correct display in IE 9-.
 */
canvas {
  display: inline-block; }

/**
 * Add the correct display in IE.
 */
template {
  display: none; }

/* Hidden
   ========================================================================== */
/**
 * Add the correct display in IE 10-.
 */
[hidden] {
  display: none; }

.u-clamp-text {
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis; }

.u-display-flex {
  display: -ms-flexbox;
  display: flex; }

.u-flex-grow {
  -ms-flex-positive: 1;
      flex-grow: 1; }

.u-font-size-small {
  font-size: 12px; }

.u-font-size-large {
  font-size: 16px; }

.u-line-height-initial {
  line-height: initial; }

.u-nudge-up {
  bottom: 1px;
  position: relative; }

.u-push-bottom {
  margin-bottom: 15px; }

.u-push-bottom-third {
  margin-bottom: 5px; }

.u-push-top {
  margin-top: 15px; }

.u-push-top-third {
  margin-top: 5px; }

.u-text-align-left {
  text-align: left; }

.u-text-align-right {
  text-align: right; }

.u-text-color-deemphasize {
  color: #949a9f; }

.u-text-color-error {
  color: #ff5d4e; }

/*# sourceMappingURL=app.ada508f9934857d427a3.css.map*/